/**
dco/c++ v3.7.1
    -- Algorithmic Differentiation by Operator Overloading in C++

COPYRIGHT 2022
The Numerical Algorithms Group Limited and
Software and Tools for Computational Engineering @ RWTH Aachen University

This file is part of dco/c++.
**/

#include "g.hpp"

template <typename DCO_TYPE> void a1s_g(DCO_TYPE& pa, const DCO_TYPE& xv, DCO_TYPE& xa) { pa = xa / (2 * xv); }

void g_make_gap(const DCO_TYPE& p, DCO_TYPE& x) {

  g(dco::value(p), dco::value(x));
  DCO_MODE::global_tape->register_variable(x);

  auto fill_gap = [=]() {
    DCO_MODE::value_t pa = 0;
    a1s_g(pa, dco::value(x), dco::derivative(x));
    dco::derivative(p) += pa;
  };

  DCO_MODE::global_tape->insert_callback(std::move(fill_gap));
}
