/**
dco/c++ v3.7.1
    -- Algorithmic Differentiation by Operator Overloading in C++

COPYRIGHT 2022
The Numerical Algorithms Group Limited and
Software and Tools for Computational Engineering @ RWTH Aachen University

This file is part of dco/c++.
**/

#include "g.hpp"

void g_make_gap(int n, DCO_TYPE& xg) {
  static int c = 1;
  int ts = c++;

  DCO_TYPE xg_in(xg);

  g(n, dco::value(xg));
  DCO_MODE::global_tape->register_variable(xg);

  auto fill_gap = [=]() {
    DCO_TYPE x = xg_in;
    DCO_TAPE_POSITION_TYPE p = DCO_MODE::global_tape->get_position();
    g(n, x);
    cerr << "ts" << ts << "=" << dco::size_of(DCO_MODE::global_tape) << "MB" << endl;
    dco::derivative(x) = dco::derivative(xg);
    DCO_MODE::global_tape->interpret_adjoint_and_reset_to(p);
  };
  DCO_MODE::global_tape->insert_callback(std::move(fill_gap));
}
